package FullSmoke::JobController;

use Moose::Role;
use Function::Parameters qw(:std :modifiers);
use FullSmoke::Util::JobController::Phase;
#use FullSmoke::Util::MooseHelpers;

# TODO use Moose::Exporter for these?
use Moose::Exporter;

Moose::Exporter->setup_import_methods(
    with_meta => [ qw/phase rolephase add_rolephase/ ],
);

has _job_mgr => (
  is => 'ro',
  isa => 'FullSmoke::JobManager',
  handles => {make_job => 'make_job'}, # so we can force our instance in it
);

has _phases => (
  is => 'ro',
  isa => 'ArrayRef[FullSmoke::Util::JobController::Phase]',
  default => sub {[]},
  traits => ['Array'],
  handles => {add_phase => 'push'},
);

has _rolephases => (
  is => 'ro',
  isa => 'HashRef[FullSmoke::Util::JobController::Phase]',
  default => sub {+{}},
  traits => ['Hash'],
  handles => {_add_rolephase => 'set'},
);

fun phase($phase, %options) {
  my ($package) = caller();
  my $phase = FullSmoke::Util::JobController::Phase->new(name => $phase, %options);

  $package->instance->add_phase($phase);
}

fun rolephase($phase) {
  my ($package) = caller();
  $package->instance->add_phase($package->instance->_rolephases->{$phase});
}

method add_rolephase($name, %options) {
  my $phase = FullSmoke::Util::JobController::Phase->new(name => $name, %options);

  $self->_add_rolephase($name, $phase);  
}

1;