package FullSmoke::JobController::Roles::BuildPerl;

use Moose::Role;
use Function::Parameters qw/:std :modifiers/;
use FullSmoke::JobController; # get the exports too
use FullSmoke::Util::MooseHelpers;

with 'FullSmoke::JobController';

# Becomes a command line option
has_str_arg perl_gitref => (
  documentation => 'Git refspec to checkout and build from',
  required => 0,
);

has_str_arg perl_git_mirror => (
  required => 0,
  documentation => 'Where to clone from, default is https://github.com/perl5/perl',
);

has_arrstr_arg build_opt => (
  required => 0,
  documentation => 'Options to pass to perlbrew, --build_opt -D'
);

# Hidden from the command line, can't set an object anyway
has _perl_job => (
  isa => 'FullSmoke::Jobs::BuildPerl',
  is => 'rw',
  handles => [qw/perl_name/],
  accessor => 'perl_job'
);

method BUILD(@args) {
  $self->add_rolephase(build_perl => (
    fatal => 1, # If any job fails, then we fail the whole build
    code => method() {
      my $perl_build = $self->make_job(
        BuildPerl => 
          git_ref => $self->perl_gitref, 
          perlbrew_opts => $self->build_opt);

      return $perl_build;
    }
  ));

  $self->add_rolephase(cleanup_perl => (
    fatal => 1,
    code => method() {
      my @jobs;

      push @jobs, $self->make_job(
        UninstallPerl =>
          perl_name => $self->_perl_job->perl_name,
      );

      return @jobs;
    }
  ));
}
1;