package FullSmoke::Jobs::BuildPerl;

use Moose;
use warnings;
with 'FullSmoke::JobRole';
use Function::Parameters;
use Path::Tiny;
use IPC::Run;
use FullSmoke::Util::MooseHelpers;

has perlbrew_args => (
  traits => ['Array'],
  is => 'ro',
  default => sub {[]},
  isa => 'ArrayRef[Str]',
  handles => {
    push_arg => 'push'
  }
);

# Name for the job manager to store this under
method canonical_name() {$self->perl_name}
has_str_arg perl_name => (
  lazy => 1,
  builder => '_make_perlname'
);

has_str_arg git_ref => ();
file_output versioninfo => ();

method _make_perlname() {
  die "Not starting, can't build name" unless $self->is_starting();

  my $name = "fullsmoke-".$self->git_ref;

  if ($self->perlbrew_args->@*) {
    $name .= join('_', map {s/\W/-/r} $self->perlbrew_args->@*);
  }

  return $name;
}

method setup() {
  # TODO do I need to do anything?
  # This method should declare dependencies
  $self->log("Setting up build perl job for: ", $self->git_ref, 
      " as ", $self->perl_name, 
      " with args [",join(' ', $self->perlbrew_args->@*),"]");
}

method cmdrun(@cmd) {
  my $cmdname = $cmd[0];
  my $output_str = "";
  my $output = fun($data) {
    for my $line (split "\n", $data) {
      $self->log("$cmdname: $line");
    }
    $output_str .= $data;
  };

  $self->log("Running [",join(' ',@cmd),"]");

  IPC::Run::run \@cmd, ">&", $output;
  my $err = $?;
  die "Failed command: $err" if $err;
  return $output_str;
}

method run() {
  my $dh = Path::Tiny->tempdir(TEMPLATE => "fullsmoke-git-XXXXXXX");
  my $tempdir = $dh->child("perl")->stringify;
  $self->log("Cloning perl git into ", $tempdir);

  $self->cmdrun(qw(/usr/bin/git clone https://www.github.com/perl/perl5),$tempdir);
  chdir $tempdir or die "Couldn't chdir($tempdir): $!";
  $self->log("Checkout out ", $self->git_ref);
  $self->cmdrun(qw(/usr/bin/git checkout), $self->git_ref);
  $self->log("Fudging for pending-authors.t");
  $self->cmdrun(qw(/usr/bin/git config --local user.name), '');
  $self->cmdrun(qw(/usr/bin/git config --local user.email), '');

  $self->log("Building perl with perlbrew");

  my @cmd = (qw[perlbrew install --verbose --as], $self->perl_name, $tempdir, $self->perlbrew_args->@*);

  $self->cmdrun(@cmd);

  $self->file_output_versioninfo(
    $self->cmdrun(qw[perlbrew exec --with], $self->perl_name, qw[perl -V])
  );

  return 1;
}

1;
