package FullSmoke::Jobs::CPANModule;

use Moose;
use warnings;
with 'FullSmoke::JobRole';
use Function::Parameters;
use Path::Tiny;
use IPC::Run;
use FullSmoke::Util::CPANIndex qw/mod2dist distinfo/;
use FullSmoke::Util::MooseHelpers;

use MooseX::Getopt::GLD;
with 'MooseX::Getopt::GLD' => { getopt_conf => [ 'pass_through'] };

file_output install => ();

has_str_arg version => ();
has_str_arg author => ();
has_str_arg perl_name => ();
has_str_arg dist_file => ();
has_str_arg dist_path => (
  lazy => 1,
  builder => '_make_dist_path',
);
method _make_dist_path() {
  return $self->author() . "/" . $self->dist_file();
}
method canonical_name() {return $self->perl_name . ":" . $self->dist_path()}

method cpanm_cmd(@cmd) {
    my @cmd = (qw/perlbrew exec --with/, $perlbrew_env, 'cpanm', @cmd);

    my $out;
    my $in='';
    $|++;

    my $timeout = 600; # fixed timeout for every command

    my $h = harness \@cmd, '<', \$in, '>&', \$out, timeout($timeout); # timeout after 10 minutes
    eval {
      $h->start();
      my $st = time();
      while($h->pumpable()) { # still getting output
        $h->pump_nb();
        if (length($out) > 10*1024*1024) { # 20 meg limit on output
          die "Output too long.  Failing build\n";
        }
        sleep(0.1);
        printf "%03.03f %08d\r", time()-$st, length($out);
        die "Timeout" if time()-$st > $timeout;
      }
    
      $h->finish();
    };
    my $err = $@;
    $h->kill_kill(); # reap all the kids
    eval {$self->log("FullSmoke: had to double kill"); $h->finish();} if $err;
    my $exitcode = $err || $h->full_result();

    return ($exitcode, $out);
}

method setup() {
  my ($exitcode, $output) = $self->cpanm_cmd("--showdeps", $self->dist_path);

  die "Failed to get deps" if $exitcode;

  my @mod_deps = map {
      my ($module, $version) = split('~', $_); 
      +{module => $module, version => $version}
    } split(/\n/, $output);
  
  for my $mod_dep (@mod_deps) {
    # TODO check versions?
    my %distinfo = distinfo mod2dist $mod_dep->{module};

    $self->register_dependency(job_type => 'CPANModule', perl_name => $self->perl_name, %distinfo);
  }
}

method run() {
  my ($exitcode, $output) = $self->cpanm_cmd(qw/cpanm --verbose/, $self->dist_path);

  $self->file_output_install($output);
  die "Install failed: $exitcode" if $exitcode;
}

1;
