package FullSmoke::Util::CPANIndex;

use Moose;
# This probably could just be some local variables, but I think I'm going to add
# more functionality in the future
use MooseX::Singleton;
use Function::Parameters qw(:std :modifiers);
use LWP::UserAgent; # Need this to be synchronous anyway, and need many protocol support
use Exporter qw/import/;
use Path::Tiny;

our @EXPORT_OK = qw/mod2dist dist2mods distinfo get_all_dists/;

has _mod2dist => (
  is => 'rw',
  isa => 'HashRef',
  default => sub {+{}}
);

has _dist2mods => (
  is => 'rw',
  isa => 'HashRef',
  default => sub {+{}}
);

has _distinfo => (
  is => 'rw',
  isa => 'HashRef',
  default => sub {+{}}
);

fun get_index() {
  my $mirror = mirror();

  my $ua = LWP::UserAgent->new();
  # TODO saner setup with URI module doing path concat
  my $response = $ua->get($mirror."/modules/02packages.details.txt");

  if ($response->is_success) {
    return $response->decoded_content;
  };
}

fun mirror() {
  # TODO add any other env variables we want to listen to?
  if (my $opts = $ENV{PERL_CPANM_OPTS}) {
    my ($mirror) = ($opts =~ /-(?:M|-mirror|-from)\s+(\S+)/);
    return $mirror;
  }

  return "http://www.cpan.org/";
}

has _built_index => (
  reader => undef,
  writer => undef,
  lazy => 1,
  is => 'ro',
  default => method() {
    my $index = get_index();
    
    $index =~ s/\A.*?\r?\n\r?\n//s; # cut off the header
    print "Getting cpan index\n"; # TODO real loggin

    for my $line (split /\r?\n/, $index) {
      if ($line =~ /^(?<module>\S+)\s+(?<version>\S+)\s+(?<dist_path>\S+)\s*$/) {
        my $obj = {module => $+{module}, version => $+{version}, dist_path => $+{dist_path}};

        my $path = Path::Tiny->new($obj->{dist_path});

        $obj->{dist_file} = $path->basename;
        $obj->{author} = $path->parent->basename;
        
        # strip off the version and past to get the dist name
        # Might not work on everything, but it seems to.
        $obj->{dist_name} = $obj->{dist_file} =~ s|-[^-]+$||r;

        $self->_mod2dist()->{$obj->{module}} = $obj->{dist_name};
        $self->_dist2mods()->{$obj->{dist_name}} = [] unless $self->_dist2mods()->{$obj->{dist_name}};
        push $self->_dist2mods()->{$obj->{dist_name}}->@*, $obj->{module};
        $self->_distinfo()->{$obj->{dist_name}} = $obj;
      } else {
        warn "Got invalid line, $line";
      }
    }

    print "Got cpan index\n";
    return 1;    
  }
);

fun mod2dist($module) {
  my $obj = __PACKAGE__->instance;
  $obj->_built_index();
  return $obj->_mod2dist()->{$module};
}

fun dist2mods($dist) {
  my $obj = __PACKAGE__->instance;
  $obj->_built_index();
  return $obj->_dist2mods()->{$dist};
}

fun distinfo($dist) {
  my $obj = __PACKAGE__->instance;
  $obj->_built_index();
  return $obj->_distinfo()->{$dist};
}

fun get_all_dists() {
  my $obj = __PACKAGE__->instance;
  $obj->_built_index();
  return $obj->_distinfo()->%*;
}

FullSmoke::Util::CPANIndex->instance;