package FullSmoke::Util::MooseHelpers;

use Function::Parameters;
use Moose ();
use Moose::Exporter;
use MooseX::Getopt::GLD;
use Syntax::Keyword::Try;

use Moose::Meta::Role;
use Class::MOP;

Moose::Exporter->setup_import_methods(
    with_meta => [ qw/has_str_arg has_int_arg has_arrstr_arg file_output/ ],
);

sub init_meta {
  my ($package, %options) = @_;

  my $rolemeta = Class::MOP::class_of("MooseX::Getopt::GLD");
  my $meta = Class::MOP::class_of($options{for_class});

  $rolemeta->apply($meta, getopt_conf => ['pass_through']);

  if ($meta->isa('Moose::Meta::Role')) {
    return Moose::Role->init_meta( %options );
  } else {
    return Moose->init_meta( %options );
  }

}

fun has_str_arg($meta, $prop, %options) {
  $meta->add_attribute($prop,
    is => 'ro',
    isa => 'Str',
    required => 1,
    predicate => "has_$prop",
    %options
  );
}

fun has_int_arg($meta, $prop, %options) {
  $meta->add_attribute($prop,
    is => 'ro',
    isa => 'Int',
    required => 1,
    predicate => "has_$prop",
    %options
  );
}

fun has_arrstr_arg($meta, $prop, %options) {
  $meta->add_attribute($prop,
    is => 'ro',
    isa => 'ArrayRef[Str]',
    required => 1,
    predicate => "has_$prop",
    %options
  );
}

fun file_output($meta, $filename, %options) {
  $meta->add_attribute("_output_$filename",
    is => 'rw',
    isa => 'Str',
    init_arg => undef,
    predicate => "_has_output_$filename",
    writer => "file_output_$filename",
    %options,
  );
}

1;